@@ -1,8 +1,15 @@
+import fcntl
+import functools
+import glob
+import operator
 import os
 import subprocess
+import time
 from typing import List, Optional, Union
 
-from . import EDID, __cache__, _monitor_brand_lookup, filter_monitors
+from . import filter_monitors, get_methods
+from .helpers import EDID, __cache__, _monitor_brand_lookup
+import warnings
 
 
 class SysFiles:
@@ -83,19 +90,12 @@ class SysFiles:
                         # to 'actual' values
                         continue
 
-                name, serial = EDID.parse_edid(device['edid'])
-                if name is not None:
-                    device['name'], device['serial'] = name, serial
-
-                    try:
-                        manufacturer_id, manufacturer = _monitor_brand_lookup(name.split(' ')[0])
-                    except TypeError:
-                        device['manufacturer'] = name.split(' ')[0]
-                        device['manufacturer_id'] = None
-                    else:
-                        device['manufacturer_id'], device['manufacturer'] = manufacturer_id, manufacturer
-
-                    device['model'] = name.split(' ')[1]
+                device.update(
+                    zip(
+                        ('manufacturer_id', 'manufacturer', 'model', 'name', 'serial'),
+                        EDID.parse(device['edid'])
+                    )
+                )
 
                 displays[device['edid']] = device
                 index += 1
@@ -128,7 +128,7 @@ class SysFiles:
             primary_brightness = sbc.linux.SysFiles.get_brightness(display = 0)[0]
 
             # get the brightness of the secondary display
-            edp_brightness = sbc.linux.SysFiles.get_brightness(display = 1)[0]
+            secondary_brightness = sbc.linux.SysFiles.get_brightness(display = 1)[0]
             ```
         '''
         info = cls.get_display_info()
@@ -153,7 +153,7 @@ class SysFiles:
 
         Args:
             value (int): Sets the brightness to this value
-            display (int): The specific display you wish to query.
+            display (int): The specific display you wish to adjust.
 
         Example:
             ```python
@@ -166,7 +166,7 @@ class SysFiles:
             sbc.linux.SysFiles.set_brightness(75, display = 0)
 
             # set the secondary display brightness to 25%
-            sbc.linux.SysFiles.set_brightness(75, display = 1)
+            sbc.linux.SysFiles.set_brightness(25, display = 1)
             ```
         '''
         info = cls.get_display_info()
@@ -178,6 +178,345 @@ class SysFiles:
                 f.write(str(int(value * device['scale'])))
 
 
+class I2C:
+    '''
+    In the same spirit as `SysFiles`, this class serves as a way of getting
+    display information and adjusting the brightness without relying on any
+    3rd party software.
+
+    Usage of this class requires read and write permission for `/dev/i2c-*`.
+
+    This class works over the I2C bus, primarily with desktop monitors as I
+    haven't tested any e-DP displays yet.
+
+    Massive thanks to [siemer](https://github.com/siemer) for
+    his work on the [ddcci.py](https://github.com/siemer/ddcci) project,
+    which served as a my main reference for this.
+
+    References:
+        * [ddcci.py](https://github.com/siemer/ddcci)
+        * [DDCCI Spec](https://milek7.pl/ddcbacklight/ddcci.pdf)
+    '''
+    # vcp commands
+    GET_VCP_CMD = 0x01
+    '''VCP command to get the value of a feature (eg: brightness)'''
+    GET_VCP_REPLY = 0x02
+    '''VCP feature reply op code'''
+    SET_VCP_CMD = 0x03
+    '''VCP command to set the value of a feature (eg: brightness)'''
+
+    # addresses
+    DDCCI_ADDR = 0x37
+    '''DDC packets are transmittred using this I2C address'''
+    HOST_ADDR_R = 0x50
+    '''Packet source address (the computer) when reading data'''
+    HOST_ADDR_W = 0x51
+    '''Packet source address (the computer) when writing data'''
+    DESTINATION_ADDR_W = 0x6e
+    '''Packet destination address (the monitor) when writing data'''
+    I2C_SLAVE = 0x0703
+    '''The I2C slave address'''
+
+    # timings
+    WAIT_TIME = 0.05
+    '''How long to wait between I2C commands'''
+
+    class I2CDevice():
+        '''
+        Class to read and write data to an I2C bus,
+        based on the `I2CDev` class from [ddcci.py](https://github.com/siemer/ddcci)
+        '''
+        def __init__(self, fname: str, slave_addr: int):
+            '''
+            Args:
+                fname (str): the I2C path, eg: `/dev/i2c-2`
+                slave_addr (int): not entirely sure what this is meant to be
+            '''
+            self.device = os.open(fname, os.O_RDWR)
+            # I2C_SLAVE address setup
+            fcntl.ioctl(self.device, I2C.I2C_SLAVE, slave_addr)
+
+        def read(self, length: int) -> bytes:
+            '''
+            Read a certain number of bytes from the I2C bus
+
+            Args:
+                length (int): the number of bytes to read
+
+            Returns:
+                bytes
+            '''
+            return os.read(self.device, length)
+
+        def write(self, data: bytes) -> int:
+            '''
+            Writes data to the I2C bus
+
+            Args:
+                data (bytes): the data to write
+
+            Returns:
+                int: the number of bytes written
+            '''
+            return os.write(self.device, data)
+
+    class DDCInterface(I2CDevice):
+        '''
+        Class to send DDC (Display Data Channel) commands to an I2C device,
+        based on the `Ddcci` and `Mccs` classes from [ddcci.py](https://github.com/siemer/ddcci)
+        '''
+
+        PROTOCOL_FLAG = 0x80
+
+        def __init__(self, i2c_path: str):
+            '''
+            Args:
+                i2c_path (str): the path to the I2C device, eg: `/dev/i2c-2`
+            '''
+            super().__init__(i2c_path, I2C.DDCCI_ADDR)
+
+        def write(self, *args) -> int:
+            '''
+            Write some data to the I2C device.
+
+            It is recommended to use `setvcp` to set VCP values on the DDC device
+            instead of using this function directly.
+
+            Args:
+                *args: variable length list of arguments. This will be put
+                    into a `bytearray` and wrapped up in various flags and
+                    checksums before being written to the I2C device
+
+            Returns:
+                int: the number of bytes that were written
+            '''
+            time.sleep(I2C.WAIT_TIME)
+
+            ba = bytearray(args)
+            ba.insert(0, len(ba) | self.PROTOCOL_FLAG)  # add length info
+            ba.insert(0, I2C.HOST_ADDR_W)  # insert source address
+            ba.append(functools.reduce(operator.xor, ba, I2C.DESTINATION_ADDR_W))  # checksum
+
+            return super().write(ba)
+
+        def setvcp(self, vcp_code: int, value: int) -> int:
+            '''
+            Set a VCP value on the device
+
+            Args:
+                vcp_code (int): the VCP command to send, eg: `0x10` is brightness
+                value (int): what to set the value to
+
+            Returns:
+                int: the number of bytes written to the device
+            '''
+            return self.write(I2C.SET_VCP_CMD, vcp_code, *value.to_bytes(2, 'big'))
+
+        def read(self, amount: int) -> bytes:
+            '''
+            Reads data from the DDC device.
+
+            It is recommended to use `getvcp` to retrieve VCP values from the
+            DDC device instead of using this function directly.
+
+            Args:
+                amount (int): the number of bytes to read
+
+            Returns:
+                bytes
+
+            Raises:
+                ValueError: if the read data is deemed invalid
+            '''
+            time.sleep(I2C.WAIT_TIME)
+
+            ba = super().read(amount + 3)
+
+            # check the bytes read
+            checks = {
+                'source address': ba[0] == I2C.DESTINATION_ADDR_W,
+                'checksum': functools.reduce(operator.xor, ba) == I2C.HOST_ADDR_R,
+                'length': len(ba) >= (ba[1] & ~self.PROTOCOL_FLAG) + 3
+            }
+            if False in checks.values():
+                raise ValueError('i2c read check failed: ' + repr(checks))
+
+            return ba[2:-1]
+
+        def getvcp(self, vcp_code: int) -> int:
+            '''
+            Retrieves a VCP value from the DDC device.
+
+            Args:
+                vcp_code (int): the VCP value to read, eg: `0x10` is brightness
+
+            Returns:
+                int
+
+            Raises:
+                ValueError: if the read data is deemed invalid
+            '''
+            self.write(I2C.GET_VCP_CMD, vcp_code)
+            ba = self.read(8)
+
+            checks = {
+                'is feature reply': ba[0] == I2C.GET_VCP_REPLY,
+                'supported VCP opcode': ba[1] == 0,
+                'answer matches request': ba[2] == vcp_code
+            }
+            if False in checks.values():
+                raise ValueError('i2c read check failed: ' + repr(checks))
+
+            # min is ba[3], max is ba[4:6] and we return current value
+            return int.from_bytes(ba[6:8], 'big')
+
+    @classmethod
+    def get_display_info(cls, display: Optional[Union[int, str]] = None) -> List[dict]:
+        '''
+        Returns information about detected displays by querying the various I2C buses
+
+        Args:
+            display (str or int): [*Optional*] The monitor to return info about.
+                Pass in the serial number, name, model, interface, edid or index.
+                This is passed to `filter_monitors`
+
+        Returns:
+            list: list of dicts
+
+        Example:
+            ```python
+            import screen_brightness_control as sbc
+
+            # get info about all monitors
+            info = sbc.linux.I2C.get_display_info()
+            # EG output: [{'name': 'Benq GL2450H', 'model': 'GL2450H', 'manufacturer': 'BenQ', 'edid': '00ffff...'}]
+
+            # get info about the primary monitor
+            primary_info = sbc.linux.I2C.get_display_info(0)[0]
+
+            # get info about a monitor called 'Benq GL2450H'
+            benq_info = sbc.linux.I2C.get_display_info('Benq GL2450H')[0]
+            ```
+        '''
+        all_displays = __cache__.get('i2c_display_info')
+        if all_displays is None:
+            all_displays = []
+            index = 0
+
+            for i2c_path in glob.glob('/dev/i2c-*'):
+                if not os.path.exists(i2c_path):
+                    continue
+
+                try:
+                    # open the I2C device using the host read address
+                    device = cls.I2CDevice(i2c_path, cls.HOST_ADDR_R)
+                    # read some 512 bytes from the device
+                    data = device.read(512)
+                except IOError:
+                    continue
+
+                # search for the EDID header within our 512 read bytes
+                start = data.find(bytes.fromhex('00 FF FF FF FF FF FF 00'))
+                if start < 0:
+                    continue
+
+                # grab 128 bytes of the edid
+                edid = data[start: start + 128]
+                # parse the EDID
+                manufacturer_id, manufacturer, model, name, serial = EDID.parse(edid)
+                # convert edid to hex string
+                edid = ''.join(f'{i:02x}' for i in edid)
+
+                all_displays.append(
+                    {
+                        'name': name,
+                        'model': model,
+                        'manufacturer': manufacturer,
+                        'manufacturer_id': manufacturer_id,
+                        'serial': serial,
+                        'method': cls,
+                        'index': index,
+                        'edid': edid,
+                        'i2c_bus': i2c_path
+                    }
+                )
+                index += 1
+
+            if all_displays:
+                __cache__.store('i2c_display_info', all_displays, expires=2)
+
+        if display is not None:
+            return filter_monitors(display=display, haystack=all_displays, include=['i2c_bus'])
+        return all_displays
+
+    @classmethod
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
+        '''
+        Gets the brightness for a display by querying the I2C bus
+
+        Args:
+            display (int): The specific display you wish to query.
+
+        Returns:
+            list: list of ints (0 to 100)
+
+        Example:
+            ```python
+            import screen_brightness_control as sbc
+
+            # get the current display brightness
+            current_brightness = sbc.linux.I2C.get_brightness()
+
+            # get the brightness of the primary display
+            primary_brightness = sbc.linux.I2C.get_brightness(display = 0)[0]
+
+            # get the brightness of the secondary display
+            secondary_brightness = sbc.linux.I2C.get_brightness(display = 1)[0]
+            ```
+        '''
+        all_displays = cls.get_display_info()
+        if display is not None:
+            all_displays = [all_displays[display]]
+
+        results = []
+        for device in all_displays:
+            interface = cls.DDCInterface(device['i2c_bus'])
+            results.append(interface.getvcp(0x10))
+
+        return results
+
+    @classmethod
+    def set_brightness(cls, value: int, display: Optional[int] = None):
+        '''
+        Sets the brightness for a display by writing to the I2C bus
+
+        Args:
+            value (int): Set the brightness to this value
+            display (int): The specific display you wish to adjust.
+
+        Example:
+            ```python
+            import screen_brightness_control as sbc
+
+            # set the brightness to 50%
+            sbc.linux.I2C.set_brightness(50)
+
+            # set the primary display brightness to 75%
+            sbc.linux.I2C.set_brightness(75, display = 0)
+
+            # set the secondary display brightness to 25%
+            sbc.linux.I2C.set_brightness(25, display = 1)
+            ```
+        '''
+        all_displays = cls.get_display_info()
+        if display is not None:
+            all_displays = [all_displays[display]]
+
+        for device in all_displays:
+            interface = cls.DDCInterface(device['i2c_bus'])
+            interface.setvcp(0x10, value)
+
+
 class Light:
     '''collection of screen brightness related methods using the light executable'''
 
@@ -254,7 +593,7 @@ class Light:
             sbc.linux.Light.set_brightness(75, display = 0)
 
             # set the secondary display brightness to 25%
-            sbc.linux.Light.set_brightness(75, display = 1)
+            sbc.linux.Light.set_brightness(25, display = 1)
             ```
         '''
         info = cls.get_display_info()
@@ -307,14 +646,29 @@ class Light:
 
 
 class XBacklight:
-    '''collection of screen brightness related methods using the xbacklight executable'''
+    '''
+    DEPRECATED.
+
+    A collection of screen brightness related methods using the xbacklight executable'''
 
     executable: str = 'xbacklight'
     '''the xbacklight executable to be called'''
 
+    @staticmethod
+    def __warn_deprecated():
+        warnings.warn(
+            (
+                'xbacklight brightness method is deprecated and will be removed in the next release.'
+                ' Please use sysfiles or i2c or light or xrandr or ddcutil instead'
+            ),
+            DeprecationWarning
+        )
+
     @classmethod
     def set_brightness(cls, value: int, **kwargs):
         '''
+        DEPRECATED.
+
         Sets the screen brightness to a supplied value
 
         Args:
@@ -329,11 +683,14 @@ class XBacklight:
             sbc.linux.XBacklight.set_brightness(100)
             ```
         '''
+        cls.__warn_deprecated()
         subprocess.call([cls.executable, '-set', str(value)])
 
     @classmethod
     def get_brightness(cls, **kwargs) -> int:
         '''
+        DEPRECATED.
+
         Returns the screen brightness as reported by xbacklight
 
         Returns:
@@ -350,6 +707,7 @@ class XBacklight:
             current_brightness = sbc.linux.XBacklight.get_brightness()
             ```
         '''
+        cls.__warn_deprecated()
         result = subprocess.check_output([cls.executable, '-get']).decode()
         if not result:
             raise ValueError('no valid output was received from xbacklight')
@@ -437,20 +795,12 @@ class XRandr:
                 )
                 tmp_display['edid'] = edid
 
-                # get name and serial from edid
-                name, serial = EDID.parse_edid(edid)
-                if name is not None:
-                    tmp_display['name'] = name
-                    tmp_display['manufacturer'] = name.split(' ')[0]
-                    tmp_display['model'] = name.split(' ')[1]
-                    tmp_display['serial'] = serial
-                    try:
-                        (
-                            tmp_display['manufacturer_id'],
-                            tmp_display['manufacturer']
-                        ) = _monitor_brand_lookup(tmp_display['manufacturer'])
-                    except TypeError:
-                        tmp_display['manufacturer_id'] = None
+                tmp_display.update(
+                    zip(
+                        ('manufacturer_id', 'manufacturer', 'model', 'name', 'serial'),
+                        EDID.parse(edid)
+                    )
+                )
 
             elif 'Brightness:' in line and brightness:
                 tmp_display['brightness'] = int(float(line.replace('Brightness:', '')) * 100)
@@ -645,8 +995,6 @@ class DDCUtil:
                     # the split() removes extra spaces
                     name = line.replace('Model:', '').split()
                     try:
-                        # EG: DELL U2211H -> Dell U2211H
-                        name[0] = name[0].lower().capitalize()
                         tmp_display['model'] = name[1]
                     except IndexError:
                         pass
@@ -756,7 +1104,8 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
     Lists detailed information about all detected monitors
 
     Args:
-        method (str): the method the monitor can be addressed by. Can be 'xrandr' or 'ddcutil' or 'light'
+        method (str): the method the monitor can be addressed by. See `screen_brightness_control.get_methods`
+            for more info on available methods
         allow_duplicates (bool): whether to filter out duplicate displays (displays with the same EDID) or not
 
     Returns:
@@ -788,30 +1137,25 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
             print('Method:', monitor['method'])
         ```
     '''
-    info = __cache__.get('linux_monitors_info', method=method, allow_duplicates=allow_duplicates)
-    if info is None:
-        methods = [XRandr, DDCUtil, Light, SysFiles]
-        if method is not None:
-            method = method.lower()
-            if method not in ('xrandr', 'ddcutil', 'light', 'sysfiles'):
-                raise ValueError((
-                    'method must be "xrandr" or "ddcutil" or "light"'
-                    ' or "sysfiles" to get monitor information'
-                ))
+    all_methods = get_methods()
 
-        info = []
-        edids = []
-        for m in methods:
-            if method is None or method == m.__name__.lower():
-                # to make sure each display (with unique edid) is only reported once
-                try:
-                    tmp = m.get_display_info()
-                except Exception:
-                    pass
-                else:
-                    for i in tmp:
-                        if allow_duplicates or i['edid'] not in edids:
-                            edids.append(i['edid'])
-                            info.append(i)
-        __cache__.store('linux_monitors_info', info, method=method, allow_duplicates=allow_duplicates)
+    if method is not None:
+        method = method.lower()
+        if method not in all_methods:
+            raise ValueError(f'method must be one of: {list(all_methods)}')
+
+    info = []
+    edids = []
+    for method_name, method_class in all_methods.items():
+        if method is None or method == method_name:
+            # to make sure each display (with unique edid) is only reported once
+            try:
+                tmp = method_class.get_display_info()
+            except Exception:
+                pass
+            else:
+                for i in tmp:
+                    if allow_duplicates or i['edid'] not in edids:
+                        edids.append(i['edid'])
+                        info.append(i)
     return info
